@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #if 0
 static char *help_pages[] = {
@@ -46,13 +47,12 @@ static char *help_pages[] = {
 
 static char *state_title[] = {
 	N_("Welcome to Bug Buddy"),
-	N_("Debugging Information"),
-	N_("Product"),
-	N_("Component"),
+	N_("Select a Product"),
+	N_("Select a Component"),
 	N_("Frequently Reported Bugs"),
 	N_("Bug Description"),
-	N_("System Configuration"),
-	N_("Submitting the Report"),
+	N_("Mail Configuration"),
+	N_("Confirmation"),
 	N_("Finished!"),
 	NULL
 };
@@ -92,14 +92,13 @@ on_druid_about_clicked (GtkWidget *button, gpointer data)
 	};
 
 	if (about) {
-		gdk_window_show (about->window);
-		gdk_window_raise (about->window);
+		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
 		
 	about = gnome_about_new (_("Bug Buddy"), VERSION,
-				 "Copyright (C) 1999, 2000, 2001 Jacob Berkman\n"
-				 "Copyright 2000, 2001 Ximian, Inc.",
+				 "Copyright (C) 1999, 2000, 2001, 2002, Jacob Berkman\n"
+				 "Copyright 2000, 2001, 2002 Ximian, Inc.",
 				 _("The graphical bug reporting tool for GNOME."),
 				 authors,
 				 documentors,
@@ -116,7 +115,8 @@ on_druid_about_clicked (GtkWidget *button, gpointer data)
 			    href, FALSE, FALSE, 0);
 	gtk_widget_show (href);
 #endif
-
+	gtk_window_set_transient_for (GTK_WINDOW (about),
+				      GTK_WINDOW (GET_WIDGET ("druid-window")));
 	gtk_widget_show (about);
 }
 
@@ -139,26 +139,22 @@ druid_set_state (BuddyState state)
 	oldstate = druid_data.state;
 	druid_data.state = state;
 
-	gtk_widget_set_sensitive (GET_WIDGET ("druid-prev"),
-				  (state > 0));
+	druid_set_sensitive ((state > 0), (state < STATE_FINISHED), TRUE);
 
-	gtk_widget_set_sensitive (GET_WIDGET ("druid-next"),
-				  (state < STATE_FINISHED));
-
-	gnome_canvas_item_set (druid_data.banner,
-			       "text", _(state_title[state]),
-			       NULL);
+	{
+		GtkLabel *label = GTK_LABEL (GET_WIDGET ("druid-label"));
+		char *s = g_strconcat ("<span size=\"xx-large\" weight=\"ultrabold\">",
+				       _(state_title[state]), "</span>", NULL);
+		gtk_label_set_label (label, s);
+		gtk_label_set_use_markup (label, TRUE);
+		g_free (s);
+	}
 
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (GET_WIDGET ("druid-notebook")),
 				       state);	
 
 	switch (druid_data.state) {
-	case STATE_INTRO:
-		/* check what gdb'ing we are going to do, and print
-		 * the right message */
-		break;
 	case STATE_GDB:
-		start_gdb ();
 		break;
 	case STATE_PRODUCT:
 #if 0
@@ -167,7 +163,9 @@ druid_set_state (BuddyState state)
 		buddy_set_text ("bts-package-entry".
 				druid_data.package_name);
 #endif
+#if 0
 		load_bugzilla_xml ();
+#endif
 #if 0
 		if (!druid_data.product)
 			druid_set_sensitive (TRUE, FALSE,  TRUE);
@@ -180,23 +178,30 @@ druid_set_state (BuddyState state)
 #endif
 		break;
 	case STATE_MOSTFREQ:
+		/* nothing to do */
+		break;		
 	case STATE_DESC:
 		/* nothing to do */
 		break;
-	case STATE_SYSTEM:
 #if 0
+	case STATE_SYSTEM:
 		/* start the process of version checking if we haven't
 		 * run anything or the list of thingies has changed */
 		do_dependency_stuff ();
-#endif
 		druid_set_state (state - 1);
 		break;
+#endif
+	case STATE_EMAIL_CONFIG:
+		/* FIXME: change next icon */
+		on_email_group_toggled (NULL, NULL);
+		break;
 	case STATE_EMAIL:
 		/* fill in the content text */
-		s = generate_email_text ();
+		s = generate_email_text (TRUE);
 		w = GET_WIDGET ("email-text");
 		buddy_set_text ("email-text", s);
 		g_free (s);
+		on_email_group_toggled (NULL, NULL);
 		break;
 	case STATE_FINISHED:
 		/* print a summary yo */
@@ -214,7 +219,11 @@ druid_set_state (BuddyState state)
 void
 on_druid_prev_clicked (GtkWidget *w, gpointer data)
 {
-	druid_set_state (druid_data.state - 1);
+	if (druid_data.state == STATE_DESC && 
+	    GTK_TOGGLE_BUTTON (GET_WIDGET ("no-product-toggle"))->active)
+		druid_set_state (STATE_PRODUCT);
+	else
+		druid_set_state (druid_data.state - 1);
 }
 
 
@@ -223,7 +232,7 @@ email_is_valid (const char *addy)
 {
 	char *rev;
 
-	if (!addy || strlen (addy) < 4 || !strchr (addy, '@') || strstr (addy, "@."))
+	if (!addy || strlen (addy) < 4 || !strchr (addy, '@') || strstr (addy, "@.") || strstr (addy, "root@"))
 		return FALSE;
 
 	g_strreverse (rev = g_strdup (addy));
@@ -262,11 +271,13 @@ email_is_valid (const char *addy)
 
 /* return true if page is ok */
 static gboolean
-intro_page_ok (void)
+mail_config_page_sendmail_ok (void)
 {
 	GtkWidget *w;
 	gchar *s;
 
+	/* FIXME: check mail config type */
+
 	s = buddy_get_text ("email-name-entry");
 	if (! (s && strlen (s))) {
 		g_free (s);
@@ -301,26 +312,20 @@ intro_page_ok (void)
 
 	s = buddy_get_text ("email-sendmail-entry");
 	if (!g_file_test (s, G_FILE_TEST_EXISTS)) {
-		if (druid_data.submit_type == SUBMIT_TO_SELF ||
-		    druid_data.submit_type == SUBMIT_REPORT) {
+		if (druid_data.submit_type == SUBMIT_SENDMAIL) {
 			w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
 						    0,
-						    GTK_MESSAGE_QUESTION,
-						    GTK_BUTTONS_YES_NO,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
 						    _("'%s' doesn't seem to exist.\n\n"
-						      "You won't be able to actually "
-						      "submit a bug report, but you will\n"
-						      "be able to save it to a file.\n\n"
-						      "Specify a new location for sendmail?"),
+						      "Please check the path to sendmail again."),
 						    s);
 			gtk_dialog_set_default_response (GTK_DIALOG (w),
-							 GTK_RESPONSE_YES);
-			if (GTK_RESPONSE_NO != gtk_dialog_run (GTK_DIALOG (w))) {
-				g_free (s);
-				gtk_widget_destroy (w);
-				return FALSE;
-			}
+							 GTK_RESPONSE_OK);
+			gtk_dialog_run (GTK_DIALOG (w));
+			g_free (s);
 			gtk_widget_destroy (w);
+			return FALSE;
 		}
 	}
 	g_free (s);
@@ -328,6 +333,47 @@ intro_page_ok (void)
 	return TRUE;
 }
 
+static gboolean
+mail_config_page_gnome_ok (void)
+{
+	GtkWidget *w;
+	char *s;
+
+	if (GTK_TOGGLE_BUTTON (GET_WIDGET ("email-default-radio"))->active)
+		return TRUE;
+
+	s = buddy_get_text ("email-command-entry");
+	if (! (s && strlen (s))) {
+		g_free (s);
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    0,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Please enter a valid email command."));
+		gtk_dialog_set_default_response (GTK_DIALOG (w),
+						 GTK_RESPONSE_OK);
+		gtk_dialog_run (GTK_DIALOG (w));
+		gtk_widget_destroy (w);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+mail_config_page_ok (void)
+{
+	switch (druid_data.submit_type) {
+	case SUBMIT_GNOME_MAILER:
+		return mail_config_page_gnome_ok ();
+	case SUBMIT_SENDMAIL:
+		return mail_config_page_sendmail_ok ();
+	case SUBMIT_FILE:
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static gboolean
 text_is_sensical (const gchar *text, int sensitivity)
 {
@@ -453,9 +499,7 @@ submit_ok (void)
 		gtk_widget_destroy (w);
 	}
 
-	to = buddy_get_text (druid_data.submit_type == SUBMIT_TO_SELF 
-			     ? "email-email-entry"
-			     : "email-to-entry");
+	to = buddy_get_text ("email-to-entry");
 
 	if (druid_data.submit_type == SUBMIT_FILE) {
 		file = buddy_get_text ("email-file-entry");
@@ -511,13 +555,6 @@ submit_ok (void)
 	}
 
 	fprintf (fp, "To: %s\n", to);
-
-	if (druid_data.submit_type == SUBMIT_REPORT &&
-	    GTK_TOGGLE_BUTTON (GET_WIDGET ("email-cc-toggle"))->active) {
-		s = buddy_get_text ("email-email-entry");
-		fprintf (fp, "Cc: %s\n", s);
-		g_free (s);
-	}
 	
 	s = buddy_get_text ("email-cc-entry");
 	if (*s) fprintf (fp, "Cc: %s\n", s);
@@ -574,18 +611,37 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 	newstate = druid_data.state + 1;
 
 	switch (druid_data.state) {
-	case STATE_INTRO:
-		/* validate email and sendmail */
-		if (!intro_page_ok ())
-			return;
-		break;
 	case STATE_GDB:
 		/* nothing */
 		break;
 	case STATE_PRODUCT:
+	{
+		BugzillaProduct *product;
 		/* check that the package is ok */
-		druid_data.product = get_selected_row ("product-list", PRODUCT_DATA);
-		if (!druid_data.product) {
+		if (GTK_TOGGLE_BUTTON (GET_WIDGET ("no-product-toggle"))->active) {
+			static gboolean dialog_shown = FALSE;
+			if (!dialog_shown) {
+				d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+							    0,
+							    GTK_MESSAGE_WARNING,
+							    GTK_BUTTONS_OK,
+							    _("Since Bug Buddy doesn't know about the product "
+							      "you wish to submit a bug report in, you will have "
+							      "to manually address the bug report."));
+				gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_OK);
+				gtk_dialog_run (GTK_DIALOG (d));
+				gtk_widget_destroy (d);
+				dialog_shown = TRUE;
+			}
+			buddy_set_text ("email-to-entry", "");
+			druid_data.product = NULL;
+			druid_data.component = NULL;
+			newstate = STATE_DESC;
+			break;
+		}
+ 
+		product = get_selected_row ("product-list", PRODUCT_DATA);
+		if (!product) {
 			d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
 						    0,
 						    GTK_MESSAGE_ERROR,
@@ -597,12 +653,18 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 			gtk_widget_destroy (d);
 			return;
 		}
-		bugzilla_product_add_components_to_clist (druid_data.product);
-		buddy_set_text ("email-to-entry", druid_data.product->bts->email);
+		if (product != druid_data.product) {
+			druid_data.product = product;
+			bugzilla_product_add_components_to_clist (druid_data.product);
+			buddy_set_text ("email-to-entry", druid_data.product->bts->email);
+		}
 		break;
+	}
 	case STATE_COMPONENT:
-		druid_data.component = get_selected_row ("component-list", COMPONENT_DATA);
-		if (!druid_data.component) {
+	{
+		BugzillaComponent *component;
+		component = get_selected_row ("component-list", COMPONENT_DATA);
+		if (!component) {
 			d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
 						    0,
 						    GTK_MESSAGE_ERROR,
@@ -629,20 +691,82 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 			return;
 		}
 		g_free (s);
-		bugzilla_add_mostfreq (druid_data.product->bts);
+		if (component != druid_data.component) {
+			druid_data.component = component;
+			bugzilla_add_mostfreq (druid_data.product->bts);
+		}
 		break;
+	}
 	case STATE_MOSTFREQ:
-		/* nothing */
 		break;
 	case STATE_DESC:
 		/* validate subject, description, and file name */
 		if (!desc_page_ok ())
 			return;
-		/* skip system */
-		newstate++;
 		break;
-	case STATE_SYSTEM:
-		/* nothing */
+	case STATE_EMAIL_CONFIG:
+		if (!mail_config_page_ok ())
+			return;
+
+		if (GTK_TOGGLE_BUTTON (GET_WIDGET ("email-mailer-radio"))->active) {
+			MailerItem *mailer;
+			char *orig_body, *uri_body, *to, *orig_subject, *uri_subject;
+			int argc;
+			char **argv;
+
+			if (GTK_TOGGLE_BUTTON (GET_WIDGET ("email-custom-radio"))->active) {
+				mailer = &druid_data.custom_mailer;
+			} else {
+				char *s;
+				s = buddy_get_text ("email-default-entry");
+				mailer = g_hash_table_lookup (druid_data.mailer_hash, s);
+				g_free (s);
+			}
+
+			/* FIXME: validate mailer */
+
+			g_shell_parse_argv (mailer->command, &argc, &argv, NULL);
+			argv = g_realloc (argv, ++argc);
+
+			/* escape from la */
+			orig_body  = generate_email_text (FALSE);
+			uri_body   = gnome_vfs_escape_string (orig_body);
+
+			orig_subject = buddy_get_text ("desc-subject");
+			uri_subject  = gnome_vfs_escape_string (orig_subject);
+
+			if (druid_data.product)
+				to = gnome_vfs_escape_string (druid_data.product->bts->email);
+			else
+				to = g_strdup ("");
+
+			argv[argc-1] = g_strdup_printf ("mailto:%s?subject=%s&body=%s", to, uri_subject, uri_body);
+			argv[argc]   = NULL;
+
+			{
+				char **s;
+				for (s = argv; *s; s++)
+					g_print ("%s\n", *s);
+			}
+
+			/* FIXME: check for errors */
+			g_spawn_async (NULL, argv, NULL,
+				       G_SPAWN_SEARCH_PATH,
+				       NULL, NULL, NULL, NULL);
+
+			g_strfreev (argv);
+			g_free (orig_body);
+			g_free (uri_body);
+			g_free (orig_subject);
+			g_free (uri_subject);
+			g_free (to);
+
+			newstate = STATE_FINISHED;
+
+			buddy_set_text ("finished-label", 
+					_("Your email program has been launched.  Please look it over and send it.\n\n"
+					  "Thank you for submitting this bug report."));
+		}
 		break;
 	case STATE_EMAIL:
 		/* validate included file.
